#ifndef BITFIELDS_H
#define BITFIELDS_H

#include <sys/param.h>
#ifndef DARWINBUILD
 #include <endian.h>		/* Use the system's endian header */
#endif
#include <stdint.h>

// Bit Field Macros
// 
// These macros allow for the correct setting and extracting of bit fields
// from the given size word.
#if __BYTE_ORDER == __LITTLE_ENDIAN
#define SWAB32(word) (word)
#define SWAB64(word) (word)
#elif __BYTE_ORDER == __BIG_ENDIAN
// From http://stackoverflow.com/questions/105252/how-do-i-convert-between-big-endian-and-little-endian-values-in-c
//#define SWAB32(word) (__builtin_bswap32(word))
//#define SWAB64(word) (__builtin_bswap64(word))
/// Old methods
/// These "by-hand" methods were written by Ron Rechenmacher
#define SWAB32(word) ( (((word) & 0xFF000000) >> 24) | (((word) & 0x00FF0000) >> 8) | (((word) & 0x0000FF00) << 8 | (((word) & 0x000000FF) << 24)) )
#define SWAB64(word) ( SWAB32( (uint32_t)((word)>>32) ) | (uint64_t)SWAB32( (uint32_t)(word) )<<32 )
#endif




// #if __BYTE_ORDER == __LITTLE_ENDIAN
// #warning "Using Little Endian Byte Order for Unpacking"
#define BITFIELDGET(block, mask, shift)        ((SWAB32(block )& mask) >> shift)
#define BITFIELDSET(block, data, mask, shift)  block = SWAB32((SWAB32(block) & ~mask)| ((data << shift ) & mask))

#define BITFIELDGET_NOSWAB(block, mask, shift)        (((block )& mask) >> shift)
#define BITFIELDSET_NOSWAB(block, data, mask, shift)  block = (((block) & ~mask)| ((data << shift ) & mask))


#endif /* BITFIELDS_H */
