#ifndef RAWDCMCHAN_H
#define RAWDCMCHAN_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/Macros.h"
#include "DAQDataFormats/BitFields.h"

#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawDCMChan
#define  CURRENT_FIELDS_SUB_NAMESPACE rawmillidcm

//! Class to hold the data from the FEBs in correct formats
  /*!
   * The RawDCMChan class is for storing DCm ID information
   * Currently, there is no versioning for this class
   *
  */

namespace daqdataformats {

  static const uint32_t RAWDCMCHAN_SIZE = 1;
  
  namespace CURRENT_FIELDS_SUB_NAMESPACE {

        enum DCMChanMASKS{
            MASK_DiblockId   = 0x00FF, // Diblock ID
            MASK_DCMId       = 0x1F00, // DCM ID
            MASK_DetId       = 0xE000, // Detector ID
            MASK_GlobalDCMID = 0xFFFF  // Global ID
        };

        enum DCMChanWORDS{
            WORD_DiblockId   = 0, // Diblock ID
            WORD_DCMId       = 0, // DCM ID
            WORD_DetId       = 0, // Detector ID
            WORD_GlobalDCMID = 0  // Global ID
        };

        DEFINE_SHIFT_FROM_MASK(DiblockId)
        DEFINE_SHIFT_FROM_MASK(DCMId)
        DEFINE_SHIFT_FROM_MASK(DetId)
        DEFINE_SHIFT_FROM_MASK(GlobalDCMID)

  }// end of  rawmillidcm
  
  class RawDCMChan : public RawDAQData {
    
        public:
    
            // Constructors
            RawDCMChan();                     //! The default constructor
            RawDCMChan(const RawDCMChan& copy_in);  //! Deep copy constructor
            RawDCMChan(const void* dcmchan_buffer); //! Shallow copy constructor
            /// Destructor
            ~RawDCMChan();

            using    RawDAQData::print;
            void     print           (std::ostream& os=std::cout) const;
            uint32_t sizeofdata      () const {return RAWDCMCHAN_SIZE;}

            IMPLEMENT_GET_AND_SET_NOVERSION(DiblockId, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(DCMId, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(DetId, uint32_t)
            IMPLEMENT_GET_AND_SET_NOVERSION(GlobalDCMID, uint32_t)
            
        protected:
            /// Read the RawDCMChan version is meaningless at this point, because we don't have versioning for it
            version_t figureOutTheVersion(const void*) const {return 0;}

            bool setFunctionPointers() {return true;}

  };
  
  
} // END NAMESPCE daqdataformats
#endif /* RAWDCMCHAN_H */
