#ifndef RAWMILLISLICEINDEX_H
#define RAWMILLISLICEINDEX_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/BitFields.h"
#include "DAQDataFormats/Macros.h"
#include "DAQDataFormats/RawMilliSliceIndexHeader.h"

#include "DAQDataFormats/DataFormatFirst.inc"
#define VERSION_NAMESPACE            RawMilliSliceIndex
#define CURRENT_FIELDS_SUB_NAMESPACE rawmilliindx


namespace daqdataformats {

    static const uint32_t DEFAULT_MILLISLICEINDEX_MAXIMUM_SIZE = 2000;
    
    namespace CURRENT_FIELDS_SUB_NAMESPACE {
            static const uint32_t  MILLISLICEINDEX_SIZE = 2;

            enum MilliSliceIndexMASKS{
                MASK_Time   = 0xFFFFFFFF, // Time length since the MilliSlice start time
                MASK_Offset = 0x000FFFFF  // Offset
            };// end of MilliSliceIndexMASKS

            enum MilliSliceIndexWORDS{
                WORD_Time   = 0,
                WORD_Offset = 1
            };// end of MilliSliceIndexWORDS

            DEFINE_SHIFT_FROM_MASK(Time)
            DEFINE_SHIFT_FROM_MASK(Offset)
    }

    class RawMilliSliceIndex: public RawDAQData {
    
        public:
    
            // Constructors
            RawMilliSliceIndex(const uint32_t size = DEFAULT_MILLISLICEINDEX_MAXIMUM_SIZE);             //! The default constructor
            RawMilliSliceIndex(const RawMilliSliceIndex& copy_in); //! Deep copy constructor
            RawMilliSliceIndex(const void* millisliceheader_buffer); //! Shallow copy constructor
            /// Destructor
            ~RawMilliSliceIndex();

            void     init            ();
            using    RawDAQData::print;
            void     print           (std::ostream& os=std::cout) const;
            uint32_t sizeofdata      () const;
            bool     setBufferSource(const void*);
            using    RawDAQData::readData;
            void*    readData(const void* buffer);

            daqdataformats::RawMilliSliceIndexHeader* getHeader() const { return _Header;}

            bool setTime  (const uint32_t, const uint32_t);
            bool setOffset(const uint32_t, const uint32_t);

            uint32_t getTime  (const uint32_t) const;
            uint32_t getOffset(const uint32_t) const;

            bool     addIndex(const uint32_t, const uint32_t);

            bool     setClosed(const bool);
            bool     setIndexNumber(const uint32_t);

            bool     advanceIndexNumber();

            bool     close();

            uint32_t getIndexNumber() const;
            bool     isClosed() const;

            uint32_t getSizeOfOneIndex() const {return CURRENT_FIELDS_SUB_NAMESPACE::MILLISLICEINDEX_SIZE;}

        private:

            version_t figureOutTheVersion(const void*) const {return 0;}
            bool      setFunctionPointers()                  {return true;}



        protected:
            daqdataformats::RawMilliSliceIndexHeader* _Header;      ///< RawMilliSliceIndexHeader

    };
  
  
} // END NAMESPCE daqdataformats
#endif /* RAWMILLISLICEINDEX_H */
