#ifndef RAWSUMMARYBLOCKHEADERV0_H
#define RAWSUMMARYBLOCKHEADERV0_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/Macros.h"
#include "DAQDataFormats/BitFields.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawSummaryBlockHeader
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE summaryblockheader
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {
          namespace CURRENT_FIELDS_SUB_NAMESPACE {
            static const uint32_t SummaryBlock_Marker = 0xFABC;
          };

          namespace  VERSION_NAMESPACE {
            namespace CURRENT_FIELDS_SUB_NAMESPACE {

                static const uint32_t SUMMARYBLOCKHEADER_SIZE = 5;

                enum SummaryBlockHeaderMASKS{
                    MASK_Marker                   = 0xFFFF0000, //! Marker Mask                        (16 bits)
                    MASK_Version                  = 0x00000FFF, //! Version Mask                       (12 bits)
                    MASK_DataSize                 = 0xFFFFFFFF, //! Data Size Mask                     (32 bits)
                    MASK_BuffId                   = 0x0000FFFF, //! Buffer Node ID Mask                (16 bits)
                    MASK_TrigNumLo                = 0xFFFFFFFF, //! Trigger Number Mask Low            (32 bits)
                    MASK_TrigNumHi                = 0x000000FF, //! Trigger Number Mask High           (8 bits)
                    MASK_MissingData              = 0x08000000, //! Is DataBlock Missing Data?         (1 bit)
                    MASK_MonteCarloFlag           = 0x10000000, //! SIM flag                           (1 bit)
                    MASK_CRCCalculationUsed       = 0x20000000, //! Is CRC calculation used?           (1 bit)
                    MASK_ConnectionInitialization = 0x40000000, //! Initialization Connection bit Mask (1 bit)
                    MASK_CloseConnection          = 0x80000000  //! Close Connection bit               (1 bit)
                };
        
                enum SummaryBlockHeaderWORDS{
                    WORD_Version                  = 0,
                    WORD_Marker                   = 0,
                    WORD_DataSize                 = 1,
                    WORD_BuffId                   = 2,
                    WORD_TrigNumLo                = 3,
                    WORD_TrigNumHi                = 4,
                    WORD_MissingData              = 4,
                    WORD_MonteCarloFlag           = 4,
                    WORD_CRCCalculationUsed       = 4,
                    WORD_ConnectionInitialization = 4,
                    WORD_CloseConnection          = 4
                };

                DEFINE_SHIFT_FROM_MASK(Version)
                DEFINE_SHIFT_FROM_MASK(Marker)
                DEFINE_SHIFT_FROM_MASK(DataSize)
                DEFINE_SHIFT_FROM_MASK(BuffId)
                DEFINE_SHIFT_FROM_MASK(TrigNumLo)
                DEFINE_SHIFT_FROM_MASK(TrigNumHi)
                DEFINE_SHIFT_FROM_MASK(MissingData)
                DEFINE_SHIFT_FROM_MASK(MonteCarloFlag)
                DEFINE_SHIFT_FROM_MASK(CRCCalculationUsed)
                DEFINE_SHIFT_FROM_MASK(ConnectionInitialization)
                DEFINE_SHIFT_FROM_MASK(CloseConnection)
        }// end of namespace summaryblockheader
      
    class RawSummaryBlockHeader: public PREVIOUS_VERSION_CLASS_NAME {
        public:
            RawSummaryBlockHeader (const version_t); ///< The default constructor
            RawSummaryBlockHeader (const RawSummaryBlockHeader&);
            ~RawSummaryBlockHeader(); ///< Destructor

            void     init            ();
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print           (std::ostream& os=std::cout) const;
            using    RawDAQData::readData;
            void*    readData(const void* pointer){return RawDAQData::readDataGeneral(pointer);};
            uint32_t sizeofdata      () const { return CURRENT_FIELDS_SUB_NAMESPACE::SUMMARYBLOCKHEADER_SIZE;}

    protected:

            IMPLEMENT_FIRST_GET_AND_SET(Version       , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(Marker        , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TrigNumLo     , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TrigNumHi     , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(BuffId        , uint16_t)
            IMPLEMENT_FIRST_GET_AND_SET(DataSize      , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET_BOOL(MonteCarloFlag)

            IMPLEMENT_FIRST_IS_AND_SET(CRCCalculationUsed)
            IMPLEMENT_FIRST_IS_AND_SET(ConnectionInitialization)
            IMPLEMENT_FIRST_IS_AND_SET(CloseConnection)
            IMPLEMENT_FIRST_IS_AND_SET(MissingData)

	    INIT_FIRST(getTrigNum           , const, uint64_t, )
            INIT_FIRST(setTrigNum           ,      , bool          , const uint64_t)

            INIT_FIRST(addDataSize          ,      , bool          , const uint32_t)
	    INIT_FIRST(checkMarker          , const, bool, )
    };
  
  
}} // END NAMESPACE daqdataformats
#endif /* RAWSUMMARYBLOCKHEADERV0_H */
