#ifndef DAQRAWTRIGGERHEADER_H
#define DAQRAWTRIGGERHEADER_H
#include "DAQDataFormats/RawTriggerHeaderV2.h"
#include "DAQDataFormats/TriggerDefines.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawTriggerHeader
#define  LAST_CLASS_VERSION           2
#define  CURRENT_FIELDS_SUB_NAMESPACE triggerheader
#include "DAQDataFormats/DataFormatLast.inc"

//! Class to hold the data from the global trigger header in the correct format
  /*!
    The RawTriggerHeader contains the basic information required to
    reconstruct where and how the trigger was generated.

    The class supports general initialization with an internal or external
    data buffer as well as copyin style initialization for use with
    STL containers.
  */


namespace daqdataformats {

  namespace triggerheader {
        static const version_t RAWTRIGGERHEADER_LATEST_VERSION = LAST_CLASS_VERSION;
  }

  class RawTriggerHeader: public PREVIOUS_VERSION_CLASS_NAME {
  
        public:
      
            RawTriggerHeader(const version_t version = triggerheader::RAWTRIGGERHEADER_LATEST_VERSION); //!< Constructor
            RawTriggerHeader(const RawTriggerHeader& copy_in);	    //!< Copyin Constructor
            RawTriggerHeader(const void* buffer, const version_t version = rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);  //!< Constructor with data intialization
            ~RawTriggerHeader();                                    //!< Destructor
    
            IMPLEMENT_MAIN_STANDARD

            IMPLEMENT_GET_MAIN        (Delimiter              , uint32_t);
            IMPLEMENT_GET_MAIN        (Version                , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(HWFlag                 , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(Source                 , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(SourceID               , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(SourceSubID            , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerNumberLow       , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerNumberHigh      , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(MasterTriggerNumberLow , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(MasterTriggerNumberHigh, uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(MasterTriggerNumber    , uint64_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerNumber          , uint64_t)

            
            IMPLEMENT_FIGUREOUT_VERSION_STATIC(RawTriggerHeader)
            
       protected:
            IMPLEMENT_SET_MAIN(Delimiter, uint32_t)
            IMPLEMENT_SET_MAIN(Version  , uint32_t)
            bool             setFunctionPointers();
           

  };
  
  
} // END NAMESPCE daqdataformats
#endif /* DAQRAWTRIGGERHEADER_H */
