/*
 * File:   QualityCheck.h
 * Author: mmathis
 *
 * Created on June 23, 2010, 12:13 PM
 */

#ifndef _QUALITYCHECK_H
#define	_QUALITYCHECK_H

#include "DAQDataFormats/RawEvent.h"
#include "DAQDataFormats/RawRun.h"
#include "DAQDataFormats/RawRunHeader.h"
#include "DAQDataFormats/RawConfigurationBlock.h"
#include "DAQDataFormats/RawMicroSlice.h"
#include "DAQDataFormats/RawNanoSlice.h"
#include "DAQDataFormats/RawTrigger.h"

namespace daqqc {

  /**
   * Checks the resulting raw run or raw event file to ensure self-consistency.
   * All of the checks performed are documented by a latex document 
   * in the doc/ directory, as well as in Nova-doc 4900.
   */
  class QualityCheck {
  public:
    QualityCheck();
    ~QualityCheck();
    
#ifndef RAWRUN_IS_NOT_IMPLEMENTED    
    bool checkRun(daqdataformats::RawRun& run);
#endif    
    bool checkEvent(daqdataformats::RawEvent& event);

    // Setters

    // Getters

  private:

    // Private helper functions
    bool checkEvent(daqdataformats::RawEvent* event);
    bool checkRunHeaderTail(daqdataformats::RawRunHeader* header, daqdataformats::RawRunHeader* tail);
    bool checkConfigBlock(daqdataformats::RawConfigurationBlock* config);
    bool checkEventHeaderTail(daqdataformats::RawEventHeader* header, daqdataformats::RawEventTail* tail);
    bool checkTriggerBlock(daqdataformats::RawTrigger* trigger);
    bool checkDataBlock(daqdataformats::RawDataBlock* data, uint32_t idata);
    bool checkMicroSlice(daqdataformats::RawMicroSlice* micro, uint32_t idata, uint32_t imicro);
    bool checkNanoSlice(daqdataformats::RawNanoSlice* nano);


    daqdataformats::RawEvent fEvent;

#ifndef RAWRUN_IS_NOT_IMPLEMENTED    
    daqdataformats::RawRun fRun;
#endif    
    uint64_t fRunNum;
    uint64_t fSubrunNum;
    uint64_t fEventNum;
    uint64_t fTrigger;
    uint32_t fPartition;
    bool fSim;
    bool fCheckRun;
    uint64_t fEventCounter;
  };
}

#endif	/* _QUALITYCHECK_H */

