#ifndef _DBUTILS_DAQAPPMGR_APPLICATIONINSTANCE_H
#define _DBUTILS_DAQAPPMGR_APPLICATIONINSTANCE_H

#include <DatabaseUtils/DAQAppMgr/ApplicationType.h>
#include <DatabaseUtils/DAQAppMgr/Host.h>
#include <DatabaseUtils/SimpleParameterSet.h>
#include <boost/shared_ptr.hpp>
#include <string>

namespace dbutils {
namespace daqappmgr {

/**
 * Simple container class for the information that describes
 * an instance of a DAQ application.
 *
 * @author Kurt Biery
 * @version $Revision: 1.3 $ $Date: 2012/05/30 14:27:44 $
 */

class ApplicationInstance {

 public:

    // ********************
    // *** Constructors ***
    // ********************

    ApplicationInstance(const std::string& appName,
                        const std::string& typeName,
                        const std::string& hostName,
                        boost::shared_ptr<ApplicationType> typePtr =
                        boost::shared_ptr<ApplicationType>(),
                        boost::shared_ptr<Host> hostPtr =
                        boost::shared_ptr<Host>(),
                        const bool& isEnabled = true);

    ApplicationInstance(const ApplicationInstance& appInstance);

    // *****************
    // *** Accessors ***
    // *****************

    std::string getAppName() const {return _appName;}
    std::string getTypeName() const {return _typeName;}
    std::string getHostName() const {return _hostName;}
    boost::shared_ptr<ApplicationType> getTypePtr() const {
        return _typePtr;
    }
    boost::shared_ptr<Host> getHostPtr() const {
        return _hostPtr;
    }
    bool isEnabled() const {return _isEnabled;}

    template <class T>
    bool getParameter(const std::string& key, T& resultValue) const {
        if (_additionalParams.get() == 0) {return false;}
        return _additionalParams->get(key, resultValue);
    }

    // *****************
    // *** Modifiers ***
    // *****************

    void setTypePtr(boost::shared_ptr<ApplicationType> typePtr) {
        _typePtr = typePtr;
    }
    void setHostPtr(boost::shared_ptr<Host> hostPtr) {
        _hostPtr = hostPtr;
    }
    void setParameterSet(boost::shared_ptr<dbutils::
                         SimpleParameterSet> additionalParams) {
        _additionalParams = additionalParams;
    }

    // *****************
    // *** Operators ***
    // *****************

    bool operator==(const ApplicationInstance& other) const;
    bool operator!=(const ApplicationInstance& other) const;
    bool operator<(const ApplicationInstance& other) const;

 private:

    std::string _appName;
    std::string _typeName;
    std::string _hostName;
    boost::shared_ptr<ApplicationType> _typePtr;
    boost::shared_ptr<Host> _hostPtr;
    boost::shared_ptr<dbutils::SimpleParameterSet> _additionalParams;
    bool _isEnabled;
};

};  // end of namespace daqappmgr
};  // end of namespace dbutils

#endif
