#ifndef _DBUTILS_DAQAPPMGR_HOST_H
#define _DBUTILS_DAQAPPMGR_HOST_H

#include <string>
#include <vector>

namespace dbutils {
namespace daqappmgr {

/**
 * Simple container class for the information that describes a DAQ host.
 *
 * @author Kurt Biery
 * @version $Revision: 1.2 $ $Date: 2012/07/02 15:09:51 $
 */

class Host {

 public:

    static const std::string MANAGER_NODE_TYPE;
    static const std::string FARM_NODE_TYPE;
    static const std::string REAL_DCM_TYPE;

    // ********************
    // *** Constructors ***
    // ********************

    Host(const std::string& hostName, const std::string& hostType,
         const std::string& sshUserName = "", const bool& ppcFlag = false,
         const bool& noKerberosFlag = false);

    // *****************
    // *** Accessors ***
    // *****************

    std::string getHostName() const {return _hostName;}
    std::string getHostType() const {return _hostType;}
    std::string getSshUserName() const {return _sshUserName;}
    bool isPPC() const {return _ppcFlag;}
    bool noKerberos() const {return _noKerberosFlag;}

    // *****************
    // *** Operators ***
    // *****************

    bool operator==(const Host& other) const;
    bool operator!=(const Host& other) const;
    bool operator<(const Host& other) const;

 private:

    std::string _hostName;
    std::string _hostType;
    std::string _sshUserName;
    bool _ppcFlag;
    bool _noKerberosFlag;

};

};  // end of namespace daqappmgr
};  // end of namespace dbutils

#endif
