#ifndef _NOVADAQ_ENVVAR_CACHE_H
#define _NOVADAQ_ENVVAR_CACHE_H

#include <NovaDAQUtilities/Cache.h>
#include <NovaDAQUtilities/TimedCachePolicy.h>
#include <boost/thread/mutex.hpp>
#include <map>

namespace novadaq {

/**
 * A singleton class that serves as a cache of environmental variable values.
 *
 * @author Kurt Biery
 * @version $Revision: 1.1 $ $Date: 2010/05/04 19:15:09 $
 */

class EnvVarCache : public novadaq::Cache {

 public:

    static EnvVarCache& getInstance();

    std::string getEnvVar(std::string name);

    int getEnvVarAsLong(std::string name, long defaultValue);

    double getEnvVarAsDouble(std::string name, double defaultValue);

 protected:

    std::string _doGetEnvVar(std::string name);

    void _loadDataIntoCache();

    void _clearCachedData();

 private:

    // constructor is private
    explicit EnvVarCache(novadaq::CachePolicy policy);

    // disallow any copying
    EnvVarCache(EnvVarCache const&);  // not implemented
    void operator= (EnvVarCache const&);  // not implemented

    // the map of env var names to values
    std::map<std::string, std::string> _envVarMap;

};

};  // end of namespace novadaq

#endif
