#ifndef _NOVADAQ_XML_DESERIALIZATION_REGISTRY_H
#define _NOVADAQ_XML_DESERIALIZATION_REGISTRY_H

#include <map>
#include <string>
#include <boost/shared_ptr.hpp>
#include <NovaDAQUtilities/XMLSerializable.h>

namespace novadaq {

/**
 * A class for registering classes that derive from XMLSerializable
 * and creating instances of those classes at a later time.  This
 * class is implemented using the Singleton pattern so that we can
 * be sure that there is only one instance in an application.
 *
 * @author Kurt Biery
 * @version $Revision: 1.2 $ $Date: 2009/11/17 18:47:08 $
 */

class XMLDeserializationRegistry {

 public:

    static XMLDeserializationRegistry& getInstance();

    void registerClass(std::string className,
                       boost::shared_ptr<XMLSerializable> classInstance);

    boost::shared_ptr<XMLSerializable> createObject(std::string className);

    static std::string translateClassName(std::string originalName);

 private:

    /**
     * Default constructor.
     */
    XMLDeserializationRegistry() {};

    // disallow any copying
    XMLDeserializationRegistry(XMLDeserializationRegistry const&);
    void operator=(XMLDeserializationRegistry const&);

    /**
     * The map of class names and instances that is used as
     * the registry of supported classes.
     */
    std::map< std::string, boost::shared_ptr<XMLSerializable> > _registry;

};

};  // end of namespace novadaq

#endif
