#include <DatabaseUtils/DAQAppMgr/ApplicationInstance.h>

namespace dbutils {
namespace daqappmgr {

/**
 * Creates an instance from the specified values.
 */
ApplicationInstance::
ApplicationInstance(const std::string& appName,
                    const std::string& typeName,
                    const std::string& hostName,
                    const boost::shared_ptr<ApplicationType> typePtr,
                    const boost::shared_ptr<Host> hostPtr,
                    const bool& isEnabled) :
    _appName(appName), _typeName(typeName), _hostName(hostName),
    _typePtr(typePtr), _hostPtr(hostPtr), _isEnabled(isEnabled)
{
}

/**
 * Creates a copy.
 */
ApplicationInstance::
ApplicationInstance(const ApplicationInstance& appInstance)
{
    _appName = appInstance._appName;
    _typeName = appInstance._typeName;
    _hostName = appInstance._hostName;
    _typePtr = appInstance._typePtr;
    _hostPtr = appInstance._hostPtr;
    _isEnabled = appInstance._isEnabled;
    _additionalParams = appInstance._additionalParams;
}

/**
 * Tests if the specified instance is equal to this one.
 */
bool ApplicationInstance::
operator==(const ApplicationInstance& other) const
{
    if (other._isEnabled && ! this->_isEnabled) {return false;}
    if (! other._isEnabled && this->_isEnabled) {return false;}

    // we leave typePtr and hostPtr out of the comparison, for now...

    return (other._appName == this->_appName &&
            other._typeName == this->_typeName &&
            other._hostName == this->_hostName);
}

/**
 * Tests if the specified set is not equal to this one.
 */
bool ApplicationInstance::
operator!=(const ApplicationInstance& other) const
{
    return !(*this==other);
}

/**
 * Tests if the specified instance is less than this one.
 */
bool ApplicationInstance::
operator<(const ApplicationInstance& other) const
{
    if (this->_appName > other._appName) {return false;}
    if (this->_appName < other._appName) {return true;}
    // else _appName is equal

    if (this->_typeName > other._typeName) {return false;}
    if (this->_typeName < other._typeName) {return true;}
    // else _typeName is equal

    if (this->_hostName > other._hostName) {return false;}
    if (this->_hostName < other._hostName) {return true;}
    // else _hostName is equal

    if (! this->_isEnabled && other._isEnabled) {return true;}
    else {return false;}
}

};  // end of namespace daqappmgr
};  // end of namespace dbutils
